"""
Dummy polish dates converter to ISO format.
"""
import re


MONTH_NAMES = {
    'styczeń': 1,
    'stycznia': 1,
    'luty': 2,
    'lutego': 2,
    'marzec': 3,
    'marca': 3,
    'kwiecień': 4,
    'kwietnia': 4,
    'maj': 5,
    'maja': 5,
    'czerwiec': 6,
    'czerwca': 6,
    'lipiec': 7,
    'lipca': 7,
    'sierpień': 8,
    'sierpnia': 8,
    'wrzesień': 9,
    'września': 9,
    'październik': 10,
    'października': 10,
    'listopad': 11,
    'listopada': 11,
    'grudzień': 12,
    'grudnia': 12,
}


ALL_MONTH_NAMES_STR = "|".join(MONTH_NAMES)
DATE_REGEX = re.compile(r"\b(?P<day>[012][0-9]|3[01]|[1-9])\s*(?P<month>" + ALL_MONTH_NAMES_STR + r")\s*(?P<year>[12]\d\d\d)\b(?:\s*(?:roku|r[.]?))?", re.IGNORECASE)


def convert_polish_date_to_iso(input_string: str) -> str:
    match = DATE_REGEX.match(input_string)
    if match:
        day_str = match.group('day')
        day_out = '0' + day_str
        day_out = day_out[-2:]

        month_str = match.group('month').lower()
        month_no = MONTH_NAMES[month_str]
        month_out = '0' + str(month_no)
        month_out = month_out[-2:]

        year_str = match.group('year')
        year_out = year_str

        return f"{year_out}-{month_out}-{day_out}"

    return None


def find_polish_dates_in_text(input_text: str):
    result = []
    position = 0
    while (match := DATE_REGEX.search(input_text, position)) is not None:
        position = match.start() + 1
        result.append(match[0])
    return result



import logging
import re
from hipisejm.stenparser.transcript import SessionSpeech, SpeechInterruption, SpeechReaction
from hipisejm.utils.pdfminer_wrapper_helper import remove_all_tags


class RawSpeechParser:
    """
    Helper class to parse raw text speech
    """
    def __init__(self):
        pass

    def parse_raw_speech(self, speaker_name: str, raw_speech_txt: str) -> SessionSpeech:
        speech = SessionSpeech(speaker_name)

        raw_speech_txt = self._preprocess_raw_speech(raw_speech_txt)

        speech_raw_parts = re.split(r"([(][^)]+[)])", raw_speech_txt)
        current_txt = ""

        for part in speech_raw_parts:
            if self._can_speech_part_be_interruption_or_reaction(part):
                # in such case need to save current text
                if len(current_txt) > 0:
                    self._add_speech_text_to_speech(speech, current_txt)
                    current_txt = ""

                interruption = self._convert_speech_part_to_interruption(part)
                if interruption is not None:
                    speech.add_interruption(interruption)
                else:
                    reaction = self._convert_speech_part_to_reaction(part)
                    if reaction is not None:
                        speech.add_reaction(reaction)
                    else:
                        message = f"Unable to parse reaction or interruption: {part}"
                        logging.info("Reaction or interruption parsing error near: %s", message)
                        continue
            else:
                # probably regular text
                current_txt = current_txt + part

        if len(current_txt) > 0:
            self._add_speech_text_to_speech(speech, current_txt)


        return speech

    def _preprocess_raw_speech(self, raw_speech_txt):
        raw_speech_txt = re.sub(r'<i>\s*</i>', ' ', raw_speech_txt)
        raw_speech_txt = re.sub(r'<b>\s*</b>', ' ', raw_speech_txt)

        # remove some special parts
        raw_speech_txt = re.sub(r"Teksty\s+wystąpień\s+niewygłoszonych.*", "", raw_speech_txt)
        raw_speech_txt = re.sub(r"[*][)]", " ", raw_speech_txt)

        raw_speech_txt = self._fix_spaces(raw_speech_txt)
        return raw_speech_txt

    def _add_speech_text_to_speech(self, speech, speech_txt):
        speech_txt = re.sub(r"</?[ib]>", "", speech_txt)
        speech_txt = self._fix_spaces(speech_txt)
        if len(speech_txt) > 0:
            speech.add_speech_text(speech_txt)

    def _can_speech_part_be_interruption_or_reaction(self, part: str) -> bool:
        if len(part) < 3:
            return False

        if not (part[0] == '(' and part[-1] == ')'):
            return False

        if not '<i>' in part:
            return False

        return True

    def _convert_speech_part_to_interruption(self, part):
        match = re.match(r"^[(]<i>(?P<speaker>.*?)(?:</i>\s*:\s*|:\s*</i>|(?:</i>\s*(?P<broken_font>[A-Za-zĄŻŚŹĘĆÓŁŃ][a-zążśźęćółń]+)\s*:))(?P<text>.*)[)]$", part)
        if match:
            speaker = match.group('speaker')
            broken_font = match.group('broken_font')
            if broken_font:
                speaker = speaker + ' ' + broken_font

            interrupted_by = self._fix_interruption_by(speaker)

            interruption_text = self._fix_spaces(match.group('text'))

            # other italic tags should not occur in interruption
            # if there is italic tag, then maybe something is wrong with the parse
            if '<i>' in interruption_text:
                return None
            return SpeechInterruption(interrupted_by, interruption_text)

        return None

    def _convert_speech_part_to_reaction(self, part):
        # Workaround for such case:
        # (<i>Oklaski</i>, <i>część posłów wstaje</i>)
        part = re.sub(r"</i>\s*,\s*<i>", ", ", part)
        part = re.sub(r"</i>\s*<i>", " ", part)

        match = re.match(r"^[(]<i>(.*)</i>([^<>]*)[)]$", part)
        if match:
            reaction_txt = match.group(1)

            additional_non_italic = match.group(2)

            # if there are multiple <i> tags in parens, then possibly this is something different than
            # audience reaction
            if '<i>' in reaction_txt:
                return None

            # sometimes some parts of the reaction are not in italic
            # this is workaround for such cases:
            # (<i>Wicemarszałek wyłącza mikrofon, poseł przemawia przy wyłączonym </i>mikrofonie)
            if additional_non_italic:
                if len(additional_non_italic) > len(reaction_txt):
                    return None
                reaction_txt = f"{reaction_txt} {additional_non_italic}"

            reaction_txt = self._fix_spaces(reaction_txt)
            return SpeechReaction(reaction_txt)

        return None

    def _fix_interruption_by(self, txt: str) -> str:
        txt = remove_all_tags(txt)
        txt = self._fix_spaces(txt)
        return txt

    def _fix_spaces(self, txt):
        txt = re.sub(r"^\s+", "", txt)
        txt = re.sub(r"\s+$", "", txt)
        txt = re.sub(r"\s+", " ", txt)
        return txt



from typing import Union, List, Optional, Type
from hipisejm.stenparser.transcript import SessionTranscript, SpeechReaction, SpeechInterruption, SessionSpeech


def get_speaker_for_utt(utt: Union[str, SpeechReaction, SpeechInterruption], speech: SessionSpeech) -> Optional[str]:
    """
    Gets speaker for <utt> object of any type (if known).
    Returns None if speaker is not known (e.g. for reaction).
    """
    if isinstance(utt, str):
        return speech.speaker
    elif isinstance(utt, SpeechReaction):
        return None
    elif isinstance(utt, SpeechInterruption):
        return utt.interrupted_by_speaker
    else:
        raise ValueError(f"Unknown object type utt: {utt}")


def get_utt_text(utt: Union[str, SpeechInterruption, SpeechReaction]) -> str:
    """
    Returns text for <utt> object of any type.
    """
    if isinstance(utt, str):
        return utt
    elif isinstance(utt, SpeechReaction):
        return utt.reaction_text
    elif isinstance(utt, SpeechInterruption):
        return utt.text
    else:
        raise ValueError(f"Unknown object type utt in: {utt}")


def leave_only_specific_type_utt(
        utt_list: List[Union[str, SpeechInterruption, SpeechReaction]],
        expected_type: Type[Union[str, SpeechInterruption, SpeechReaction]],
        exclude_mode: bool = False) -> List[Union[str, SpeechInterruption, SpeechReaction]]:
    """
    Returns only utts with specific type.

    If exclude_mode is set then returns only utts which are NOT of given type
    """
    result = []
    for utt in utt_list:
        to_take = False
        if exclude_mode:
            if not isinstance(utt, expected_type):
                to_take = True
        else:
            if isinstance(utt, expected_type):
                to_take = True

        if to_take:
            result.append(utt)

    return result
